<?php if ( ! defined('BASEPATH')) exit('No direct script access allowed');

class VserverInfo extends MX_Controller {

    /**
     * Index Page for this controller.
     *
     * Past the login page this loads the page for /vserver/vserverInfo.
     *
     */

    public function index() {

        $CI =& get_instance();
        
        // We load the BlueOnyx helper library first of all, as we heavily depend on it:
        $this->load->helper('blueonyx');
        init_libraries();

        // Need to load 'BxPage' for page rendering:
        $this->load->library('BxPage');
        $MX =& get_instance();

        // Get $sessionId and $loginName from Cookie (if they are set):
        $sessionId = $CI->input->cookie('sessionId');
        $loginName = $CI->input->cookie('loginName');
        $locale = $CI->input->cookie('locale');

        // Line up the ducks for CCE-Connection:
        include_once('ServerScriptHelper.php');
        $serverScriptHelper = new ServerScriptHelper($sessionId, $loginName);
        $cceClient = $serverScriptHelper->getCceClient();
        $user = $cceClient->getObject("User", array("name" => $loginName));
        $i18n = new I18n("base-vserver", $user['localePreference']);
        $system = $cceClient->getObject("System");

        // Initialize Capabilities so that we can poll the access rights as well:
        $Capabilities = new Capabilities($cceClient, $loginName, $sessionId);

        // -- Actual page logic start:

        // Get URL strings:
        $get_form_data = $CI->input->get(NULL, TRUE);

        //
        //-- Validate GET data:
        //

        if (isset($get_form_data['group'])) {
            // We have a delete transaction:
            $vpsid = $get_form_data['group'];
        }
        else {
            // Don't play games with us!
            // Nice people say goodbye, or CCEd waits forever:
            $cceClient->bye();
            $serverScriptHelper->destructor();
            Log403Error("/gui/Forbidden403#1");
        }

        if (isset($get_form_data['stgfx'])) {
            $stgfx = $get_form_data['stgfx'];
        }
        else {
            // Don't play games with us!
            // Nice people say goodbye, or CCEd waits forever:
            $cceClient->bye();
            $serverScriptHelper->destructor();
            Log403Error("/gui/Forbidden403#2");
        }

        $subact = '';
        if (isset($get_form_data['subact'])) {
            $subact = 'fail';
        }

        // Not 'manageSite'? Bye, bye!
        if (!$Capabilities->getAllowed('ManageVPS')) {
            // Nice people say goodbye, or CCEd waits forever:
            $cceClient->bye();
            $serverScriptHelper->destructor();
            Log403Error("/gui/Forbidden403#3");
        }

        //
        //-- Prepare data:
        //

        include_once('/usr/sausalito/ui/chorizo/ci/application/modules/base/vserver/controllers/vserverLibs.php');
        $vserverLibs = new VserverLibs($serverScriptHelper, $cceClient, $loginName, $sessionId);
        $vzlist = $vserverLibs->GetConfig();

        if (!$Capabilities->getAllowed('systemAdministrator')) {
            if (!$vserverLibs->VserverAccessRights($user, $vpsid)) {
                // Nice people say goodbye, or CCEd waits forever:
                $cceClient->bye();
                $serverScriptHelper->destructor();
                Log403Error("/gui/Forbidden403#4");
            }
        }

        // Get array with our resource parameters of VPS's:
        $res_params = $vserverLibs->GetResourceParams();

        if (!in_array($stgfx, array_keys($res_params))) {
            // Nice people say goodbye, or CCEd waits forever:
            $cceClient->bye();
            $serverScriptHelper->destructor();
            Log403Error("/gui/Forbidden403#4");
        }

        //
        //--- Handle form validation:
        //

        // We start without any active errors:
        $errors = array();
        $extra_headers =array();
        $ci_errors = array();
        $my_errors = array();

        // Shove submitted input into $form_data after passing it through the XSS filter:
        $form_data = $CI->input->post(NULL, TRUE);

        // Form fields that are required to have input:
        $required_keys = array();

        // Set up rules for form validation. These validations happen before we submit to CCE and further checks based on the schemas are done:

        // Empty array for key => values we want to submit to CCE:
        $attributes = array();
        // Items we do NOT want to submit to CCE:
        $ignore_attributes = array("BlueOnyx_Info_Text");
        if (is_array($form_data)) {
            // Function GetFormAttributes() walks through the $form_data and returns us the $parameters we want to
            // submit to CCE. It intelligently handles checkboxes, which only have "on" set when they are ticked.
            // In that case it pulls the unticked status from the hidden checkboxes and addes them to $parameters.
            // It also transformes the value of the ticked checkboxes from "on" to "1". 
            //
            // Additionally it generates the form_validation rules for CodeIgniter.
            //
            // params: $i18n                i18n Object of the error messages
            // params: $form_data           array with form_data array from CI
            // params: $required_keys       array with keys that must have data in it. Needed for CodeIgniter's error checks
            // params: $ignore_attributes   array with items we want to ignore. Such as Labels.
            // return:                      array with keys and values ready to submit to CCE.
            $attributes = GetFormAttributes($i18n, $form_data, $required_keys, $ignore_attributes, $i18n);
        }
        //Setting up error messages:
        $CI->form_validation->set_message('required', $i18n->get("[[palette.val_is_required]]", false, array("field" => "\"%s\"")));        

        // Do we have validation related errors?
        if ($CI->form_validation->run() == FALSE) {

            if (validation_errors()) {
                // Set CI related errors:
                $ci_errors = array(validation_errors('<div class="alert dismissible alert_red"><img width="40" height="36" src="/.adm/images/icons/small/white/alarm_bell.png"><strong>', '</strong></div>'));
            }           
            else {
                // No errors. Pass empty array along:
                $ci_errors = array();
            }
        }

        //
        //--- Own error checks:
        //

        if ($CI->input->post(NULL, TRUE)) {
            // Suspended sites must NOT have site_preview enabled:
            if ($attributes['suspend'] == "1") { 
                $attributes['site_preview'] = "0"; 
            }

            if ($attributes['prefix'] != "") {
                $userPrefixEnabled = "1";
            }
            else {
                $userPrefixEnabled = "0";
            }

            if (!isset($attributes['createdUser'])) {
                $attributes['createdUser'] = "admin";
            }
        }

        //
        //--- At this point all checks are done. If we have no errors, we can submit the data to CODB:
        //

        // Join the various error messages:
        $errors = array_merge($ci_errors, $my_errors);

        // If we have no errors and have POST data, we submit to CODB:
        if ((count($errors) == "0") && ($CI->input->post(NULL, TRUE))) {

        }

        //
        //-- Generate page:
        //

        $cceClient->setObjectForce("VserverMasterNode", array('gui_list_id' => $vpsid, 'gui_list_single' => time()), '', array());
        $cceClient->setObjectForce("VserverTraffic", array('generate_vps_graphs' => $vpsid, 'generate_graphs_multi' => time()), '', array());

        // Prepare Page:
        $factory = $serverScriptHelper->getHtmlComponentFactory("base-vserver", "/vserver/vserverInfo?group=$vpsid");
        $BxPage = $factory->getPage();
        $BxPage->setErrors($errors);
        $i18n = $factory->getI18n();

        // Set Menu items:
        $BxPage->setVerticalMenu('base_sitemanage');
        $BxPage->setVerticalMenuChild('base_vsite_general');
        $page_module = 'base_sitemanage';

        //
        //-- VPS short header:
        //

        $serverList = array();

        $defaultPage = "basicSettingsTab";
        $block =& $factory->getPagedBlock("vserverTitle", array($defaultPage));
        $id = $vzlist[$vpsid]['runtime']['hostname'] . " (VPSID: $vpsid)";
        $block->setLabel($factory->getLabel('modVsiteStatus', false, array('site' => $id)));
        $block->setToggle("#");
        $block->setSideTabs(FALSE);
        $block->setDefaultPage($defaultPage);
        $access = 'rw';

        $scrollList = $factory->getScrollList("vserverTitle", array("vpsid", "IP-Address", "Hostname", "Status", "Procs", "Load Average", "boot", "Owner", ''), $serverList); 
        $scrollList->setAlignments(array("center", "right", "center", "center", "center", "center", "center", "center", "center"));
        $scrollList->setDefaultSortedIndex('0');
        $scrollList->setSortOrder('ascending');
        $scrollList->setSortDisabled(array('0', '1', '2', '3', '4', '5', '6', '7', '8', '9'));
        $scrollList->setPaginateDisabled(TRUE);
        $scrollList->setSearchDisabled(TRUE);
        $scrollList->setSelectorDisabled(TRUE);
        $scrollList->enableAutoWidth(FALSE);
        $scrollList->setInfoDisabled(TRUE);
        $scrollList->setColumnWidths(array("30", "120", "238", "50", "50", "95", "73", "73", "10")); // Max: 739px

        // VPS Status light:
        if ($vzlist[$vpsid]['runtime']['status'] != "running") {
            $vps_status = $factory->getStatusSignal('noMonitor');
        }
        else {
            if ($vzlist[$vpsid]['AM']['failcnt_general'] == 'R') {
                $vps_status = $factory->getStatusSignal('severeProblem');
            }
            elseif ($vzlist[$vpsid]['AM']['failcnt_general'] == 'Y') {
                $vps_status = $factory->getStatusSignal('problem');
            }
            elseif ($vzlist[$vpsid]['AM']['failcnt_general'] == 'G') {
                $vps_status = $factory->getStatusSignal('normal');
            }
            else {
                $vps_status = $factory->getStatusSignal('noMonitor');
            }
        }

        if (!isset($vzlist[$vpsid]['config']['ONBOOT'])) {
            $vzlist[$vpsid]['config']['ONBOOT'] = 'no';
        }
        // Assemble the ScrollList-Entries:
        $scrollList->addEntry(
                array(
                        $vpsid,
                        $vzlist[$vpsid]['runtime']['ip'],
                        $vzlist[$vpsid]['runtime']['hostname'],
                        $vzlist[$vpsid]['runtime']['status'],
                        $vzlist[$vpsid]['runtime']['numproc'],
                        $vzlist[$vpsid]['runtime']['laverage'],
                        $vzlist[$vpsid]['config']['ONBOOT'],
                        'admin',
                        $vps_status
                ),
                '', false, '0');

        // Push out the Scrollist:
        $block->addFormField(
            $factory->getRawHTML("vserverTitle", $scrollList->toHtml()),
            $factory->getLabel("vserverTitle"),
            $defaultPage
        );

        //
        //-- Userbean-Counters:
        //

        $bean_block =& $factory->getPagedBlock("vserver_beancount", array($defaultPage));
        $bean_block->setToggle("#");
        $bean_block->setSideTabs(FALSE);
        $bean_block->setDefaultPage($defaultPage);

        $beanList = array();
        $bean_scrollList = $factory->getScrollList("vserver_beancount", array("", "resource", "held", "maxheld", "barrier", "limit", "stats", "failcnt", "failstats"), $beanList); 
        $bean_scrollList->setAlignments(array("center", "center", "center", "center", "center", "center", "center", "center", "center"));
        $bean_scrollList->setDefaultSortedIndex('1');
        $bean_scrollList->setSortOrder('ascending');
        $bean_scrollList->setSortDisabled(array('9'));
        $bean_scrollList->setPaginateDisabled(TRUE);
        $bean_scrollList->setSearchDisabled(TRUE);
        $bean_scrollList->setSelectorDisabled(FALSE);
        $bean_scrollList->enableAutoWidth(FALSE);
        $bean_scrollList->setInfoDisabled(TRUE);
        $bean_scrollList->setColumnWidths(array("30", "140", "100", "100", "100", "95", "50", "74", "50")); // Max: 739px

        // Status-Light:
        if ($vzlist[$vpsid]['runtime']['status'] != 'running') {
            // VPS is not running, set status to noMonitor:
            $status = $factory->getStatusSignal('noMonitor');
            $setme = '1';
        }
        else {
            // VPS is running:
            if ($vzlist[$vpsid]['runtime'][$res_params[$stgfx]['4']] > '0') {
                // Error present. 
                $status = $factory->getStatusSignal('severeProblem');
                $failcnt_general = '1';
                $setme = '1';
            }
            else {
                // No error present:
                $status = $factory->getStatusSignal('normal');
                $failcnt_general = '0';
                $setme = '1';
            }
        }
        // If we get here and setme is not set or one, then we NEED a default.
        if (($setme == '0') || (!$setme)) {
            $status = $factory->getStatusSignal('noMonitor');
        }

        // Detail-Button:
        $detailButton = '<button title="' . $i18n->getWrapped("[[palette.detail_help]]") . '" class="tiny icon_only div_icon tooltip hover right link_button" data-link="/traffic/vserverInfo?group=' . $vpsid . '&stgfx=' . $stgfx . '" target="_self" formtarget="_self"><div class="ui-icon ui-icon-search"></div></button>';

        // Fail-Button:
        $failButton = '<button title="' . $i18n->getWrapped("[[palette.detail_help]]") . '" class="tiny icon_only div_icon tooltip hover right link_button" data-link="/traffic/vserverInfo?group=' . $vpsid . '&stgfx=' . $stgfx . '&subact=fail" target="_self" formtarget="_self"><div class="ui-icon ui-icon-zoomin"></div></button>';

        $barrier = simplify_number($vzlist[$vpsid]['runtime'][$res_params[$stgfx]['2']], "KB", "0", "");
        if (strlen($barrier > '6')) {
            $barrier = $i18n->get("[[base-vserver.def_unlimited]]");
        }
        $limit = simplify_number($vzlist[$vpsid]['runtime'][$res_params[$stgfx]['3']], "KB", "0", "");
        if (strlen($limit > '6')) {
            $limit = $i18n->get("[[base-vserver.def_unlimited]]");
        }

        $bean_scrollList->addEntry(array(
            $status,
            $stgfx,
            simplify_number($vzlist[$vpsid]['runtime'][$res_params[$stgfx]['0']], "KB", "0", ""),
            simplify_number($vzlist[$vpsid]['runtime'][$res_params[$stgfx]['1']], "KB", "0", ""),
            $barrier,
            $limit,
            $detailButton,
            simplify_number($vzlist[$vpsid]['runtime'][$res_params[$stgfx]['4']], "KB", "0", ""),
            $failButton
        ));

        // Push out the Scrollist:
        $bean_block->addFormField(
            $factory->getRawHTML("vserver_beancount", $bean_scrollList->toHtml()),
            $factory->getLabel("vserver_beancount"),
            $defaultPage
        );

        // Add the Back-Button:
        $bean_block->addButton($factory->getBackButton("/vserver/vserverStatus?group=$vpsid"));

        //
        //-- Gfx:
        //

        $gfx_block =& $factory->getPagedBlock("$stgfx", array($defaultPage));
        $gfx_block->setToggle("#");
        $gfx_block->setSideTabs(FALSE);
        //$gfx_block->setShowAllTabs("#");
        $gfx_block->setDefaultPage($defaultPage);

        // Hidden formfield in case we have no gfx.
        $gfx_block->addFormField(
                $factory->getTextField("vpsid", $vpsid, "r"),
                $factory->getLabel('vpsid'),
                'hidden'
                );

        $gfxpath = "/usr/sausalito/ui/web/base/traffic/vpsimages/$vpsid/" . $subact . $stgfx . "_daily.png";
        if (file_exists($gfxpath)) {

            // Add Divider:
            $gfx_block->addFormField(
                    $factory->addBXDivider("Last_Day", ""),
                    $factory->getLabel("[[base-traffic.Last_Day]]", false),
                    $defaultPage
                    );

            $insertGfx = '<p align="center"><img src="/base/traffic/vpsimages/' . $vpsid . '/' . $subact . $stgfx . '_daily.png?x" alt=""/></p>';
            $gfx_block->addFormField(
                    $factory->getRawHTML("ld", $insertGfx, "r"),
                    $factory->getLabel('ld'),
                    $defaultPage
                    );
        }
        $gfxpath = "/usr/sausalito/ui/web/base/traffic/vpsimages/$vpsid/" . $subact . $stgfx . "_weekly.png";
        if (file_exists($gfxpath)) {

            // Add Divider:
            $gfx_block->addFormField(
                    $factory->addBXDivider("Last_Week", ""),
                    $factory->getLabel("[[base-traffic.Last_Week]]", false),
                    $defaultPage
                    );

            $insertGfx = '<p align="center"><img src="/base/traffic/vpsimages/' . $vpsid . '/' . $subact . $stgfx . '_weekly.png?x" alt=""/></p>';
            $gfx_block->addFormField(
                    $factory->getRawHTML("lw", $insertGfx, "r"),
                    $factory->getLabel('lw'),
                    $defaultPage
                    );
        }
        $gfxpath = "/usr/sausalito/ui/web/base/traffic/vpsimages/$vpsid/" . $subact . $stgfx . "_monthly.png";
        if (file_exists($gfxpath)) {
            // Add Divider:
            $gfx_block->addFormField(
                    $factory->addBXDivider("Last_Month", ""),
                    $factory->getLabel("[[base-traffic.Last_Month]]", false),
                    $defaultPage
                    );

            $insertGfx = '<p align="center"><img src="/base/traffic/vpsimages/' . $vpsid . '/' . $subact . $stgfx . '_monthly.png?x" alt=""/></p>';
            $gfx_block->addFormField(
                    $factory->getRawHTML("lm", $insertGfx, "r"),
                    $factory->getLabel('lm'),
                    $defaultPage
                    );
        }
        $gfxpath = "/usr/sausalito/ui/web/base/traffic/vpsimages/$vpsid/" . $subact . $stgfx . "_yearly.png";
        if (file_exists($gfxpath)) {
            // Add Divider:
            $gfx_block->addFormField(
                    $factory->addBXDivider("Last_Year", ""),
                    $factory->getLabel("[[base-traffic.Last_Year]]", false),
                    $defaultPage
                    );

            $insertGfx = '<p align="center"><img src="/base/traffic/vpsimages/' . $vpsid . '/' . $subact . $stgfx . '_yearly.png?x" alt=""/></p>';
            $gfx_block->addFormField(
                    $factory->getRawHTML("ly", $insertGfx, "r"),
                    $factory->getLabel('ly'),
                    $defaultPage
                    );
        }

        //
        //-- Rest:
        //

        // Nice people say goodbye, or CCEd waits forever:
        $cceClient->bye();
        $serverScriptHelper->destructor();

        $page_body[] = $block->toHtml();
        $page_body[] = $bean_block->toHtml();
        $page_body[] = $gfx_block->toHtml();

        // Out with the page:
        $BxPage->render($page_module, $page_body);

    }       
}

/*
Copyright (c) 2016 Michael Stauber, SOLARSPEED.NET
All Rights Reserved.

1. Redistributions of source code must retain the above copyright 
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright 
   notice, this list of conditions and the following disclaimer in 
   the documentation and/or other materials provided with the 
   distribution.

3. Neither the name of the copyright holder nor the names of its 
   contributors may be used to endorse or promote products derived 
   from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.

You acknowledge that this software is not designed or intended for 
use in the design, construction, operation or maintenance of any 
nuclear facility.

*/
?>